/*
 * Created on 7 mai 2004
 * Created by Olivier Chalouhi
 * 
 * Copyright (C) 2004, 2005, 2006 Aelitis SAS, All rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * AELITIS, SAS au capital de 46,603.30 euros,
 * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
 */
package org.gudy.azureus2.ui.swt.update;


import com.aelitis.azureus.core.*;

import org.eclipse.swt.SWT;
import org.gudy.azureus2.core3.config.*;
import org.gudy.azureus2.core3.util.*;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.core3.logging.*;

import org.gudy.azureus2.ui.swt.MessageBoxWindow;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.mainwindow.MainWindow;
import org.gudy.azureus2.ui.swt.mainwindow.SWTThread;

import org.gudy.azureus2.update.CoreUpdateChecker;

import org.gudy.azureus2.plugins.update.*;

/**
 * @author Olivier Chalouhi
 *
 */
public class 
UpdateMonitor 
	implements UpdateCheckInstanceListener 
{
	private static final LogIDs LOGID = LogIDs.GUI;
	public static final long AUTO_UPDATE_CHECK_PERIOD = 23*60*60*1000;  // 23 hours

	protected static UpdateMonitor		singleton;
	
	protected static AEMonitor			class_mon 	= new AEMonitor( "UpdateMonitor:class" );
	
	
	public static UpdateMonitor
	getSingleton(
		AzureusCore		core,
		MainWindow		main_window )
	{
		try{
			class_mon.enter();
		
			if ( singleton == null ){
				
				singleton = new UpdateMonitor( core, main_window );
			}
			
			return( singleton );
			
		}finally{
			
			class_mon.exit();
		}
	}

	protected AzureusCore			azureus_core;
	protected UpdateWindow 			current_update_window;
	protected UpdateCheckInstance	current_update_instance;
	
	  		
	
	protected 
	UpdateMonitor(
		AzureusCore				_azureus_core ,
		final MainWindow		_main_window )
	{
		azureus_core	= _azureus_core;
		
	}
  
	protected void
	performAutoCheck(
		final boolean		start_of_day )
	{

	}
	
	public void
	performCheck(final boolean bForce)
	{

	}
	
	public void
	complete(
		UpdateCheckInstance		instance )
	{
			// we can get here for either update actions (triggered above) or for plugin
			// install actions (triggered by the plugin installer)
		
		boolean	update_action = instance.getType() == UpdateCheckInstance.UCI_UPDATE;
		
		MainWindow mainWindow = MainWindow.getWindow();
		mainWindow.setStatusText("");
		
	    Update[] us = instance.getUpdates();
	   
	    boolean	show_window = false;
	    
	    	// updates with zero-length downloaders exist for admin purposes
	    	// and shoudn't cause the window to be shown if only they exist
	    
	    for (int i=0;i<us.length;i++){
	    	
	    	if (us[i].getDownloaders().length > 0 ){
	    		
	    		show_window	= true;
	    		
	    		break;
	    	}
	    }
	    
	    	// this controls whether or not the update window is displayed
	    	// note that we just don't show the window if this is set, we still do the
	    	// update check (as amongst other things we want ot know the latest
	    	// version of the core anyway	    
	    
    	if ( show_window ){
    		
    			// don't show another update if one's already there!
    		
    		UpdateWindow	this_window = null;
    		
    		if ( update_action ){
    			
    			if ( current_update_window == null || current_update_window.isDisposed()){
    			
    				this_window = current_update_window = new UpdateWindow( azureus_core, instance );
    			}
    		}else{

    				// always show an installer window
    			
    			this_window = new UpdateWindow( azureus_core, instance );
    		}
    		
    		if ( this_window != null ){
    			
	    		for( int i = 0 ;  i < us.length; i++ ){
				
	    			if ( us[i].getDownloaders().length > 0 ){
	    				
	    				this_window.addUpdate(us[i]);
	    			}
	    		}
	    		
	    		this_window.updateAdditionComplete();
	    		
    		}else{
    			if (Logger.isEnabled())
						Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING,
								"UpdateMonitor: user dialog already "
										+ "in progress, updates skipped"));

    		}
    	}else{
    		if (Logger.isEnabled())
					Logger.log(new LogEvent(LOGID, "UpdateMonitor: check instance "
							+ "resulted in no user-actionable updates"));

    	}
	} 
	
	public void
	cancelled(
		UpdateCheckInstance		instance )
	{
		MainWindow mainWindow = MainWindow.getWindow();

		mainWindow.setStatusText("");

	}
}
